#! /usr/bin/python
# -*- coding: utf-8 -*-

import subprocess, os, sys, re
import cv2
import json
import time
from datetime import datetime
import urllib2
import urllib
import dateutil.parser
from poster.encode import multipart_encode
from poster.streaminghttp import register_openers
import argparse
import RPi.GPIO as GPIO

BUTTON = 20
LED    = 16
DEVICE = 0
CARD   = 0
VOLUME = 50

speaker  = 'haruka'
accuracy = 70
hold_time= 1.5

GPIO.setmode(GPIO.BCM)
GPIO.setup(BUTTON, GPIO.IN, pull_up_down=GPIO.PUD_UP)
GPIO.setup(LED, GPIO.OUT)

for i in range(3):
    GPIO.output(LED, GPIO.HIGH)
    time.sleep(.5)
    GPIO.output(LED, GPIO.LOW)
    time.sleep(.5)

models_cat = ['scene', 'fashion_pattern', 'fashion_type', 'fashion_style', 'fashion_color', 'food', 'flower', 'kinoko']

docomo_key='xxx'
URL_CAT   ='https://api.apigw.smt.docomo.ne.jp/imageRecognition/v1/concept/classify/?APIKEY='
URL_SPEECH='https://api.apigw.smt.docomo.ne.jp/voiceText/v1/textToSpeech?APIKEY='

def camera():
    now = datetime.now()
    dir_name = now.strftime('%Y%m%d')
    dir_path = '/home/pi/web/image/' + dir_name + '/'
    file_name= now.strftime('%H%M%S') + '.jpg'
    fname    = dir_path + file_name

    try:
        os.mkdir(dir_path)
    except OSError:
        print 'Date dir already exists'

    os.system('sudo raspistill -o ' + fname)

    return fname


def getImage(fname, modelName):
    register_openers()
    f = open(fname, 'r')
    url = URL_CAT
    datagen, headers = multipart_encode({'image': f, 'modelName': modelName})

    url = url + docomo_key
    request = urllib2.Request(url,datagen, headers)
    response= urllib2.urlopen(request)
    res_dat = response.read()

    res_json = json.loads(res_dat)['candidates']

    return res_json

def speech(message, speaker, volume):
    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
    }
    data = {
        'text': message,
        'speaker': speaker,
        'volume': VOLUME,
        'emotion': 'happiness',
        'emotion_level': 1,
        'pitch': 100,
        'speed': 100,
    }

    try:
        values = urllib.urlencode(data)
        url = URL_SPEECH + docomo_key
        req = urllib2.Request(url, values, headers)
        res = urllib2.urlopen(req)
        header = res.info()
        data   = res.read()
        code   = res.getcode()
    except Exception as e:
        print e
        exit()
    print code, header
    return code,data


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--image' ,dest='image', type=str, default='', help='name of input image')
    parser.add_argument('--model', dest='model', type=str, default='', help='modelName = {scene, fasion_pattern, fashion_type, fashion_style, fashion_color, food, flower, kinoko, bodyPart, currency}')

    args = parser.parse_args()
    model_name = args.model if args.model else ''
    fname = args.image if args.image else ''
    print model_name, fname

    GPIO.add_event_detect(BUTTON,GPIO.FALLING)
    while True:
        if GPIO.event_detected(BUTTON):
            GPIO.remove_event_detect(BUTTON)
            now = time.time()
            count = 0
            GPIO.add_event_detect(BUTTON,GPIO.RISING)
            while time.time() < now + hold_time:
                if GPIO.event_detected(BUTTON):
                    count +=1
                    time.sleep(.3) # debounce time

            print count
            if count > 0:
                GPIO.output(LED, GPIO.HIGH)
                fname = camera()
                time.sleep(.5)
                GPIO.output(LED, GPIO.LOW)

                if fname:
                    img = cv2.imread(fname)
                    oname = fname.replace('.jpg', '_o.jpg')

                    candidate_list = []
                    if model_name == '':
                        for model in models_cat:
                            candidate_list += getImage(fname, model)
                    else:
                        image_list = getImage(fname, model_name)
                        if model_name == 'word':
                            candidate_list = getWordList(image_list)
                        else:
                            candidate_list = image_list
                    print candidate_list

                    obj= 0
                    objects   = ''
                    if candidate_list:
                        for can in candidate_list:
                            score    = round(can['score']*100, 1)
                            object   = can['tag']
                            obj_model= model_name
                            if score > accuracy:
                                obj +=1
                                print object + "(" + str(score) + "%)"
                                if object: objects += object + ', '

                        message = str(obj) + " object detected: "
                        if objects:
                            message += objects.encode('utf-8') + "が写っているかもしれません！"
                    else:
                        message = "No candidate list"

                if message:
                    print message
                    code, data = speech(message, speaker, VOLUME)
                    if code == 200:
                        f=open('speech.wav', 'w')
                        f.write(data)
                        f.close()
                        os.system('aplay -D plughw:{},{} speech.wav'.format(CARD, DEVICE))
                    else:
                        print "Speech response is not 200"
                else:
                    print "No message"

    GPIO.remove_event_detect(BUTTON)
